@@ -6,24 +6,31 @@ use anyhow::Result;
 use log::debug;
 use rustpython_parser::lexer::LexResult;
 
+use settings::pyproject;
+use settings::Settings;
+
 use crate::autofix::fixer::Mode;
 use crate::linter::{check_path, tokenize};
 use crate::message::Message;
-use crate::settings::{RawSettings, Settings};
+use crate::settings::configuration::Configuration;
 
 mod ast;
 mod autofix;
 pub mod cache;
 pub mod check_ast;
 mod check_lines;
+mod check_tokens;
 pub mod checks;
+mod checks_gen;
 pub mod cli;
 pub mod code_gen;
+mod cst;
 mod docstrings;
 mod flake8_bugbear;
 mod flake8_builtins;
 mod flake8_comprehensions;
 mod flake8_print;
+mod flake8_quotes;
 pub mod fs;
 pub mod linter;
 pub mod logging;
@@ -34,7 +41,6 @@ pub mod printer;
 mod pycodestyle;
 mod pydocstyle;
 mod pyflakes;
-pub mod pyproject;
 mod python;
 mod pyupgrade;
 pub mod settings;
@@ -54,7 +60,8 @@ pub fn check(path: &Path, contents: &str) -> Result<Vec<Message>> {
         None => debug!("Unable to find pyproject.toml; using default settings..."),
     };
 
-    let settings = Settings::from_raw(RawSettings::from_pyproject(&pyproject, &project_root)?);
+    let settings =
+        Settings::from_configuration(Configuration::from_pyproject(&pyproject, &project_root)?);
 
     // Tokenize once.
     let tokens: Vec<LexResult> = tokenize(contents);
