@@ -25,10 +25,11 @@ use ruff::linter::{lint_path, lint_stdin};
 use ruff::logging::set_up_logging;
 use ruff::message::Message;
 use ruff::printer::{Printer, SerializationFormat};
-use ruff::pyproject::{self};
-use ruff::settings::CurrentSettings;
-use ruff::settings::RawSettings;
-use ruff::settings::{FilePattern, PerFileIgnore, Settings};
+use ruff::settings::configuration::Configuration;
+use ruff::settings::pyproject;
+use ruff::settings::types::{FilePattern, PerFileIgnore};
+use ruff::settings::user::UserConfiguration;
+use ruff::settings::Settings;
 use ruff::tell_user;
 
 #[cfg(feature = "update-informer")]
@@ -73,10 +74,14 @@ fn check_for_updates() {
     }
 }
 
-fn show_settings(settings: RawSettings, project_root: Option<PathBuf>, pyproject: Option<PathBuf>) {
+fn show_settings(
+    configuration: Configuration,
+    project_root: Option<PathBuf>,
+    pyproject: Option<PathBuf>,
+) {
     println!(
         "{:#?}",
-        CurrentSettings::from_settings(settings, project_root, pyproject)
+        UserConfiguration::from_configuration(configuration, project_root, pyproject)
     );
 }
 
@@ -220,7 +225,8 @@ fn autoformat(files: &[PathBuf], settings: &Settings) -> Result<usize> {
 }
 
 fn inner_main() -> Result<ExitCode> {
-    let cli = Cli::parse();
+    let mut cli = Cli::parse();
+    cli.quiet |= cli.silent;
 
     set_up_logging(cli.verbose)?;
 
@@ -255,15 +261,15 @@ fn inner_main() -> Result<ExitCode> {
         .map(|pair| PerFileIgnore::new(pair, &project_root))
         .collect();
 
-    let mut settings = RawSettings::from_pyproject(&pyproject, &project_root)?;
+    let mut configuration = Configuration::from_pyproject(&pyproject, &project_root)?;
     if !exclude.is_empty() {
-        settings.exclude = exclude;
+        configuration.exclude = exclude;
     }
     if !extend_exclude.is_empty() {
-        settings.extend_exclude = extend_exclude;
+        configuration.extend_exclude = extend_exclude;
     }
     if !per_file_ignores.is_empty() {
-        settings.per_file_ignores = per_file_ignores;
+        configuration.per_file_ignores = per_file_ignores;
     }
     if !cli.select.is_empty() {
         warn_on(
@@ -271,10 +277,10 @@ fn inner_main() -> Result<ExitCode> {
             &cli.select,
             &cli.ignore,
             &cli.extend_ignore,
-            &settings,
+            &configuration,
             &pyproject,
         );
-        settings.select = cli.select;
+        configuration.select = cli.select;
     }
     if !cli.extend_select.is_empty() {
         warn_on(
@@ -282,22 +288,22 @@ fn inner_main() -> Result<ExitCode> {
             &cli.extend_select,
             &cli.ignore,
             &cli.extend_ignore,
-            &settings,
+            &configuration,
             &pyproject,
         );
-        settings.extend_select = cli.extend_select;
+        configuration.extend_select = cli.extend_select;
     }
     if !cli.ignore.is_empty() {
-        settings.ignore = cli.ignore;
+        configuration.ignore = cli.ignore;
     }
     if !cli.extend_ignore.is_empty() {
-        settings.extend_ignore = cli.extend_ignore;
+        configuration.extend_ignore = cli.extend_ignore;
     }
     if let Some(target_version) = cli.target_version {
-        settings.target_version = target_version;
+        configuration.target_version = target_version;
     }
     if let Some(dummy_variable_rgx) = cli.dummy_variable_rgx {
-        settings.dummy_variable_rgx = dummy_variable_rgx;
+        configuration.dummy_variable_rgx = dummy_variable_rgx;
     }
 
     if cli.show_settings && cli.show_files {
@@ -305,11 +311,11 @@ fn inner_main() -> Result<ExitCode> {
         return Ok(ExitCode::FAILURE);
     }
     if cli.show_settings {
-        show_settings(settings, project_root, pyproject);
+        show_settings(configuration, project_root, pyproject);
         return Ok(ExitCode::SUCCESS);
     }
 
-    let settings = Settings::from_raw(settings);
+    let settings = Settings::from_configuration(configuration);
 
     if cli.show_files {
         show_files(&cli.files, &settings);
@@ -342,7 +348,7 @@ fn inner_main() -> Result<ExitCode> {
         tell_user!("Starting linter in watch mode...\n");
 
         let messages = run_once(&cli.files, &settings, !cli.no_cache, false)?;
-        if !cli.quiet {
+        if !cli.silent {
             printer.write_continuously(&messages)?;
         }
 
@@ -362,7 +368,7 @@ fn inner_main() -> Result<ExitCode> {
                             tell_user!("File change detected...\n");
 
                             let messages = run_once(&cli.files, &settings, !cli.no_cache, false)?;
-                            if !cli.quiet {
+                            if !cli.silent {
                                 printer.write_continuously(&messages)?;
                             }
                         }
@@ -388,13 +394,13 @@ fn inner_main() -> Result<ExitCode> {
                 let path = Path::new(&filename);
                 (
                     run_once_stdin(&settings, path, cli.fix)?,
-                    !cli.quiet && !cli.fix,
+                    !cli.silent && !cli.fix,
                     false,
                 )
             } else {
                 (
                     run_once(&cli.files, &settings, !cli.no_cache, cli.fix)?,
-                    !cli.quiet,
+                    !cli.silent,
                     !cli.quiet,
                 )
             };
